use ::libc;

pub use crate::stddef_h::size_t;
pub use crate::stdlib::__int32_t;
pub use crate::stdlib::__uint32_t;
pub use crate::stdlib::__uint8_t;
pub use crate::stdlib::int32_t;

pub use crate::squeezelite_h::_buf_used;
pub use crate::squeezelite_h::buffer;
pub use crate::squeezelite_h::fade_dir;
pub use crate::squeezelite_h::fade_mode;
pub use crate::squeezelite_h::fade_state;
pub use crate::squeezelite_h::frames_t;
pub use crate::squeezelite_h::gain;
pub use crate::squeezelite_h::lDEBUG;
pub use crate::squeezelite_h::lERROR;
pub use crate::squeezelite_h::lINFO;
pub use crate::squeezelite_h::lSDEBUG;
pub use crate::squeezelite_h::lWARN;
pub use crate::squeezelite_h::log_level;
pub use crate::squeezelite_h::output_format;
pub use crate::squeezelite_h::output_state;
pub use crate::squeezelite_h::outputstate;
pub use crate::squeezelite_h::s32_t;
pub use crate::squeezelite_h::to_gain;
pub use crate::squeezelite_h::u32_t;
pub use crate::squeezelite_h::u8_t;
pub use crate::squeezelite_h::C2RustUnnamed_1;
pub use crate::squeezelite_h::FADE_ACTIVE;
pub use crate::squeezelite_h::FADE_CROSS;
pub use crate::squeezelite_h::FADE_CROSSFADE;
pub use crate::squeezelite_h::FADE_DOWN;
pub use crate::squeezelite_h::FADE_DUE;
pub use crate::squeezelite_h::FADE_IN;
pub use crate::squeezelite_h::FADE_INACTIVE;
pub use crate::squeezelite_h::FADE_INOUT;
pub use crate::squeezelite_h::FADE_NONE;
pub use crate::squeezelite_h::FADE_OUT;
pub use crate::squeezelite_h::FADE_UP;
pub use crate::squeezelite_h::OUTPUT_BUFFER;
pub use crate::squeezelite_h::OUTPUT_OFF;
pub use crate::squeezelite_h::OUTPUT_PAUSE_FRAMES;
pub use crate::squeezelite_h::OUTPUT_RUNNING;
pub use crate::squeezelite_h::OUTPUT_SKIP_FRAMES;
pub use crate::squeezelite_h::OUTPUT_START_AT;
pub use crate::squeezelite_h::OUTPUT_STOPPED;
pub use crate::squeezelite_h::S16_LE;
pub use crate::squeezelite_h::S24_3LE;
pub use crate::squeezelite_h::S24_LE;
pub use crate::squeezelite_h::S32_LE;
pub use crate::src::buffer::_buf_cont_read;
pub use crate::src::buffer::_buf_inc_readp;
pub use crate::src::buffer::_buf_resize;
pub use crate::src::buffer::buf_destroy;
pub use crate::src::buffer::buf_flush;
pub use crate::src::buffer::buf_init;
pub use crate::src::output_alsa::test_open;
pub use crate::src::slimproto::wake_controller;
pub use crate::src::utils::gettime_ms;
pub use crate::src::utils::logprint;
pub use crate::src::utils::logtime;
pub use crate::src::utils::touch_memory;
pub use crate::stdlib::__pthread_internal_list;
pub use crate::stdlib::__pthread_list_t;
pub use crate::stdlib::__pthread_mutex_s;
pub use crate::stdlib::pthread_mutex_t;
use crate::stdlib::sprintf;
pub use crate::stdlib::u_int32_t;
pub use crate::stdlib::u_int8_t;

use crate::stdlib::exit;
use crate::stdlib::free;
use crate::stdlib::malloc;
use crate::stdlib::memset;
use crate::stdlib::pthread_mutex_lock;
use crate::stdlib::pthread_mutex_unlock;
use crate::stdlib::strcat;
/*
 *  Squeezelite - lightweight headless squeezebox emulator
 *
 *  (c) Adrian Smith 2012-2015, triode1@btinternet.com
 *      Ralph Irving 2015-2017, ralph_irving@hotmail.com
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
// Common output function

static mut loglevel: crate::squeezelite_h::log_level = crate::squeezelite_h::lERROR;
#[no_mangle]

pub static mut output: crate::squeezelite_h::outputstate = crate::squeezelite_h::outputstate {
    state: crate::squeezelite_h::OUTPUT_STOPPED,
    format: crate::squeezelite_h::S32_LE,
    device: 0 as *const libc::c_char,
    buffer: 0,
    period: 0,
    track_started: false,
    write_cb: None,
    start_frames: 0,
    frames_played: 0,
    frames_played_dmp: 0,
    current_sample_rate: 0,
    supported_rates: [0; 18],
    default_sample_rate: 0,
    error_opening: false,
    device_frames: 0,
    updated: 0,
    track_start_time: 0,
    current_replay_gain: 0,
    c2rust_unnamed: crate::squeezelite_h::C2RustUnnamed_1 { pause_frames: 0 },
    next_sample_rate: 0,
    track_start: 0 as *const crate::squeezelite_h::u8_t as *mut crate::squeezelite_h::u8_t,
    gainL: 0,
    gainR: 0,
    invert: false,
    next_replay_gain: 0,
    threshold: 0,
    fade: crate::squeezelite_h::FADE_INACTIVE,
    fade_start: 0 as *const crate::squeezelite_h::u8_t as *mut crate::squeezelite_h::u8_t,
    fade_end: 0 as *const crate::squeezelite_h::u8_t as *mut crate::squeezelite_h::u8_t,
    fade_dir: 0 as crate::squeezelite_h::fade_dir,
    fade_mode: crate::squeezelite_h::FADE_NONE,
    fade_secs: 0,
    rate_delay: 0,
    delay_active: false,
    stop_time: 0,
    idle_to: 0,
};

static mut buf: crate::squeezelite_h::buffer = crate::squeezelite_h::buffer {
    buf: 0 as *const crate::squeezelite_h::u8_t as *mut crate::squeezelite_h::u8_t,
    readp: 0 as *const crate::squeezelite_h::u8_t as *mut crate::squeezelite_h::u8_t,
    writep: 0 as *const crate::squeezelite_h::u8_t as *mut crate::squeezelite_h::u8_t,
    wrap: 0 as *const crate::squeezelite_h::u8_t as *mut crate::squeezelite_h::u8_t,
    size: 0,
    base_size: 0,
    mutex: crate::stdlib::pthread_mutex_t {
        __data: crate::stdlib::__pthread_mutex_s {
            __lock: 0,
            __count: 0,
            __owner: 0,
            __nusers: 0,
            __kind: 0,
            __spins: 0,
            __elision: 0,
            __list: crate::stdlib::__pthread_list_t {
                __prev: 0 as *const crate::stdlib::__pthread_internal_list
                    as *mut crate::stdlib::__pthread_internal_list,
                __next: 0 as *const crate::stdlib::__pthread_internal_list
                    as *mut crate::stdlib::__pthread_internal_list,
            },
        },
    },
};
#[no_mangle]

pub static mut outputbuf: *mut crate::squeezelite_h::buffer =
    unsafe { &buf as *const crate::squeezelite_h::buffer as *mut crate::squeezelite_h::buffer };
#[no_mangle]

pub static mut silencebuf: *mut crate::squeezelite_h::u8_t =
    0 as *const crate::squeezelite_h::u8_t as *mut crate::squeezelite_h::u8_t;
#[no_mangle]

pub static mut user_rates: bool = 0 as libc::c_int != 0;
// functions starting _* are called with mutex locked
#[no_mangle]

pub unsafe extern "C" fn _output_frames(
    mut avail: crate::squeezelite_h::frames_t,
) -> crate::squeezelite_h::frames_t {
    let mut frames: crate::squeezelite_h::frames_t = 0;
    let mut size: crate::squeezelite_h::frames_t = 0;
    let mut silence: bool = false;
    let mut cross_gain_in = 0 as libc::c_int;
    let mut cross_gain_out = 0 as libc::c_int;
    let mut cross_ptr = 0 as *mut crate::squeezelite_h::s32_t;
    let mut gainL = if output.current_replay_gain != 0 {
        crate::squeezelite_h::gain(
            output.gainL as crate::squeezelite_h::s32_t,
            output.current_replay_gain as crate::squeezelite_h::s32_t,
        ) as libc::c_uint
    } else {
        output.gainL
    } as crate::squeezelite_h::s32_t;
    let mut gainR = if output.current_replay_gain != 0 {
        crate::squeezelite_h::gain(
            output.gainR as crate::squeezelite_h::s32_t,
            output.current_replay_gain as crate::squeezelite_h::s32_t,
        ) as libc::c_uint
    } else {
        output.gainR
    } as crate::squeezelite_h::s32_t;
    if output.invert {
        gainL = -gainL;
        gainR = -gainR
    }
    frames =
        crate::squeezelite_h::_buf_used(outputbuf).wrapping_div(8 as libc::c_int as libc::c_uint);
    silence = 0 as libc::c_int != 0;
    // start when threshold met
    if output.state as libc::c_int == crate::squeezelite_h::OUTPUT_BUFFER as libc::c_int
        && frames
            > output
                .threshold
                .wrapping_mul(output.next_sample_rate)
                .wrapping_div(10 as libc::c_int as libc::c_uint)
        && frames > output.start_frames
    {
        output.state = crate::squeezelite_h::OUTPUT_RUNNING;
        if loglevel as libc::c_uint >= crate::squeezelite_h::lINFO as libc::c_int as libc::c_uint {
            crate::src::utils::logprint(
                b"%s %s:%d start buffer frames: %u\n\x00" as *const u8 as *const libc::c_char,
                crate::src::utils::logtime(),
                (*::std::mem::transmute::<&[u8; 15], &[libc::c_char; 15]>(b"_output_frames\x00"))
                    .as_ptr(),
                64 as libc::c_int,
                frames,
            );
        }
        crate::src::slimproto::wake_controller();
    }
    // skip ahead - consume outputbuf but play nothing
    if output.state as libc::c_int == crate::squeezelite_h::OUTPUT_SKIP_FRAMES as libc::c_int {
        if frames > 0 as libc::c_int as libc::c_uint {
            let mut skip = if frames < output.c2rust_unnamed.skip_frames {
                frames
            } else {
                output.c2rust_unnamed.skip_frames
            };
            if loglevel as libc::c_uint
                >= crate::squeezelite_h::lINFO as libc::c_int as libc::c_uint
            {
                crate::src::utils::logprint(
                    b"%s %s:%d skip %u of %u frames\n\x00" as *const u8 as *const libc::c_char,
                    crate::src::utils::logtime(),
                    (*::std::mem::transmute::<&[u8; 15], &[libc::c_char; 15]>(
                        b"_output_frames\x00",
                    ))
                    .as_ptr(),
                    72 as libc::c_int,
                    skip,
                    output.c2rust_unnamed.skip_frames,
                );
            }
            frames = (frames as libc::c_uint).wrapping_sub(skip) as crate::squeezelite_h::frames_t
                as crate::squeezelite_h::frames_t;
            output.frames_played = output.frames_played.wrapping_add(skip);
            while skip > 0 as libc::c_int as libc::c_uint {
                let mut cont_frames = if skip
                    < crate::src::buffer::_buf_cont_read(outputbuf)
                        .wrapping_div(8 as libc::c_int as libc::c_uint)
                {
                    skip
                } else {
                    crate::src::buffer::_buf_cont_read(outputbuf)
                        .wrapping_div(8 as libc::c_int as libc::c_uint)
                };
                skip = (skip as libc::c_uint).wrapping_sub(cont_frames)
                    as crate::squeezelite_h::frames_t
                    as crate::squeezelite_h::frames_t;
                crate::src::buffer::_buf_inc_readp(
                    outputbuf,
                    cont_frames.wrapping_mul(8 as libc::c_int as libc::c_uint),
                );
            }
        }
        output.state = crate::squeezelite_h::OUTPUT_RUNNING
    }
    // pause frames - play silence for required frames
    if output.state as libc::c_int == crate::squeezelite_h::OUTPUT_PAUSE_FRAMES as libc::c_int {
        if loglevel as libc::c_uint >= crate::squeezelite_h::lINFO as libc::c_int as libc::c_uint {
            crate::src::utils::logprint(
                b"%s %s:%d pause %u frames\n\x00" as *const u8 as *const libc::c_char,
                crate::src::utils::logtime(),
                (*::std::mem::transmute::<&[u8; 15], &[libc::c_char; 15]>(b"_output_frames\x00"))
                    .as_ptr(),
                86 as libc::c_int,
                output.c2rust_unnamed.pause_frames,
            );
        }
        if output.c2rust_unnamed.pause_frames == 0 as libc::c_int as libc::c_uint {
            output.state = crate::squeezelite_h::OUTPUT_RUNNING
        } else {
            silence = 1 as libc::c_int != 0;
            frames = if avail < output.c2rust_unnamed.pause_frames {
                avail
            } else {
                output.c2rust_unnamed.pause_frames
            };
            frames = if frames < 2048 as libc::c_int as libc::c_uint {
                frames
            } else {
                2048 as libc::c_int as libc::c_uint
            };
            output.c2rust_unnamed.pause_frames =
                (output.c2rust_unnamed.pause_frames as libc::c_uint).wrapping_sub(frames)
                    as crate::squeezelite_h::u32_t as crate::squeezelite_h::u32_t
        }
    }
    // start at - play silence until jiffies reached
    if output.state as libc::c_int == crate::squeezelite_h::OUTPUT_START_AT as libc::c_int {
        let mut now = crate::src::utils::gettime_ms();
        if now >= output.c2rust_unnamed.start_at
            || output.c2rust_unnamed.start_at
                > now.wrapping_add(10000 as libc::c_int as libc::c_uint)
        {
            output.state = crate::squeezelite_h::OUTPUT_RUNNING
        } else {
            let mut delta_frames = output
                .c2rust_unnamed
                .start_at
                .wrapping_sub(now)
                .wrapping_mul(output.current_sample_rate)
                .wrapping_div(1000 as libc::c_int as libc::c_uint);
            silence = 1 as libc::c_int != 0;
            frames = if avail < delta_frames {
                avail
            } else {
                delta_frames
            };
            frames = if frames < 2048 as libc::c_int as libc::c_uint {
                frames
            } else {
                2048 as libc::c_int as libc::c_uint
            }
        }
    }
    // play silence if buffering or no frames
    if output.state as libc::c_int <= crate::squeezelite_h::OUTPUT_BUFFER as libc::c_int
        || frames == 0 as libc::c_int as libc::c_uint
    {
        silence = 1 as libc::c_int != 0;
        frames = if avail < 2048 as libc::c_int as libc::c_uint {
            avail
        } else {
            2048 as libc::c_int as libc::c_uint
        }
    }
    if loglevel as libc::c_uint >= crate::squeezelite_h::lSDEBUG as libc::c_int as libc::c_uint {
        crate::src::utils::logprint(
            b"%s %s:%d avail: %d frames: %d silence: %d\n\x00" as *const u8 as *const libc::c_char,
            crate::src::utils::logtime(),
            (*::std::mem::transmute::<&[u8; 15], &[libc::c_char; 15]>(b"_output_frames\x00"))
                .as_ptr(),
            116 as libc::c_int,
            avail,
            frames,
            silence as libc::c_int,
        );
    }
    frames = if frames < avail { frames } else { avail };
    size = frames;
    while size > 0 as libc::c_int as libc::c_uint {
        let mut out_frames: crate::squeezelite_h::frames_t = 0;
        let mut cont_frames_0 = crate::src::buffer::_buf_cont_read(outputbuf)
            .wrapping_div(8 as libc::c_int as libc::c_uint);
        let mut wrote: libc::c_int = 0;
        if !output.track_start.is_null() && !silence {
            if output.track_start == (*outputbuf).readp {
                let mut delay = 0 as libc::c_int as libc::c_uint;
                if output.current_sample_rate != output.next_sample_rate {
                    delay = output.rate_delay
                }
                frames = (frames as libc::c_uint).wrapping_sub(size)
                    as crate::squeezelite_h::frames_t
                    as crate::squeezelite_h::frames_t;
                // add silence delay in two halves, before and after track start on rate or pcm-dop change
                if delay != 0 {
                    output.state = crate::squeezelite_h::OUTPUT_PAUSE_FRAMES; // first delay - don't process track start
                    if !output.delay_active {
                        output.c2rust_unnamed.pause_frames = output
                            .current_sample_rate
                            .wrapping_mul(delay)
                            .wrapping_div(2000 as libc::c_int as libc::c_uint);
                        output.delay_active = 1 as libc::c_int != 0;
                        break;
                    } else {
                        output.c2rust_unnamed.pause_frames = output
                            .next_sample_rate
                            .wrapping_mul(delay)
                            .wrapping_div(2000 as libc::c_int as libc::c_uint);
                        output.delay_active = 0 as libc::c_int != 0
                        // second delay - process track start
                    }
                }
                if loglevel as libc::c_uint
                    >= crate::squeezelite_h::lINFO as libc::c_int as libc::c_uint
                {
                    crate::src::utils::logprint(
                        b"%s %s:%d track start sample rate: %u replay_gain: %u\n\x00" as *const u8
                            as *const libc::c_char,
                        crate::src::utils::logtime(),
                        (*::std::mem::transmute::<&[u8; 15], &[libc::c_char; 15]>(
                            b"_output_frames\x00",
                        ))
                        .as_ptr(),
                        149 as libc::c_int,
                        output.next_sample_rate,
                        output.next_replay_gain,
                    );
                }
                output.frames_played = 0 as libc::c_int as libc::c_uint;
                output.track_started = 1 as libc::c_int != 0;
                output.track_start_time = crate::src::utils::gettime_ms();
                output.current_sample_rate = output.next_sample_rate;
                if output.fade as libc::c_uint
                    == crate::squeezelite_h::FADE_INACTIVE as libc::c_int as libc::c_uint
                    || output.fade_mode as libc::c_uint
                        != crate::squeezelite_h::FADE_CROSSFADE as libc::c_int as libc::c_uint
                {
                    output.current_replay_gain = output.next_replay_gain
                }
                output.track_start = 0 as *mut crate::squeezelite_h::u8_t;
                break;
            } else if output.track_start > (*outputbuf).readp {
                // reduce cont_frames so we find the next track start at beginning of next chunk
                cont_frames_0 = if (cont_frames_0 as libc::c_long)
                    < output.track_start.wrapping_offset_from((*outputbuf).readp) as libc::c_long
                        / 8 as libc::c_int as libc::c_long
                {
                    cont_frames_0 as libc::c_long
                } else {
                    (output.track_start.wrapping_offset_from((*outputbuf).readp) as libc::c_long)
                        / 8 as libc::c_int as libc::c_long
                } as crate::squeezelite_h::frames_t
            }
        }
        if output.fade as libc::c_uint != 0 && !silence {
            if output.fade as libc::c_uint
                == crate::squeezelite_h::FADE_DUE as libc::c_int as libc::c_uint
            {
                if output.fade_start == (*outputbuf).readp {
                    if loglevel as libc::c_uint
                        >= crate::squeezelite_h::lINFO as libc::c_int as libc::c_uint
                    {
                        crate::src::utils::logprint(
                            b"%s %s:%d fade start reached\n\x00" as *const u8
                                as *const libc::c_char,
                            crate::src::utils::logtime(),
                            (*::std::mem::transmute::<&[u8; 15], &[libc::c_char; 15]>(
                                b"_output_frames\x00",
                            ))
                            .as_ptr(),
                            177 as libc::c_int,
                        );
                    }
                    output.fade = crate::squeezelite_h::FADE_ACTIVE
                } else if output.fade_start > (*outputbuf).readp {
                    cont_frames_0 = if (cont_frames_0 as libc::c_long)
                        < output.fade_start.wrapping_offset_from((*outputbuf).readp) as libc::c_long
                            / 8 as libc::c_int as libc::c_long
                    {
                        cont_frames_0 as libc::c_long
                    } else {
                        (output.fade_start.wrapping_offset_from((*outputbuf).readp) as libc::c_long)
                            / 8 as libc::c_int as libc::c_long
                    } as crate::squeezelite_h::frames_t
                }
            }
            if output.fade as libc::c_uint
                == crate::squeezelite_h::FADE_ACTIVE as libc::c_int as libc::c_uint
            {
                // find position within fade
                let mut cur_f = if (*outputbuf).readp >= output.fade_start {
                    ((*outputbuf).readp.wrapping_offset_from(output.fade_start) as libc::c_long)
                        / 8 as libc::c_int as libc::c_long
                } else {
                    ((*outputbuf)
                        .readp
                        .offset((*outputbuf).size as isize)
                        .wrapping_offset_from(output.fade_start)
                        as libc::c_long)
                        / 8 as libc::c_int as libc::c_long
                } as crate::squeezelite_h::frames_t;
                let mut dur_f = if output.fade_end >= output.fade_start {
                    (output.fade_end.wrapping_offset_from(output.fade_start) as libc::c_long)
                        / 8 as libc::c_int as libc::c_long
                } else {
                    (output
                        .fade_end
                        .offset((*outputbuf).size as isize)
                        .wrapping_offset_from(output.fade_start)
                        as libc::c_long)
                        / 8 as libc::c_int as libc::c_long
                } as crate::squeezelite_h::frames_t;
                if cur_f >= dur_f {
                    if output.fade_mode as libc::c_uint
                        == crate::squeezelite_h::FADE_INOUT as libc::c_int as libc::c_uint
                        && output.fade_dir as libc::c_uint
                            == crate::squeezelite_h::FADE_DOWN as libc::c_int as libc::c_uint
                    {
                        if loglevel as libc::c_uint
                            >= crate::squeezelite_h::lINFO as libc::c_int as libc::c_uint
                        {
                            crate::src::utils::logprint(
                                b"%s %s:%d fade down complete, starting fade up\n\x00" as *const u8
                                    as *const libc::c_char,
                                crate::src::utils::logtime(),
                                (*::std::mem::transmute::<&[u8; 15], &[libc::c_char; 15]>(
                                    b"_output_frames\x00",
                                ))
                                .as_ptr(),
                                191 as libc::c_int,
                            );
                        }
                        output.fade_dir = crate::squeezelite_h::FADE_UP;
                        output.fade_start = (*outputbuf).readp;
                        output.fade_end = (*outputbuf)
                            .readp
                            .offset(dur_f.wrapping_mul(8 as libc::c_int as libc::c_uint) as isize);
                        if output.fade_end >= (*outputbuf).wrap {
                            output.fade_end = output.fade_end.offset(-((*outputbuf).size as isize))
                        }
                        cur_f = 0 as libc::c_int as crate::squeezelite_h::frames_t
                    } else if output.fade_mode as libc::c_uint
                        == crate::squeezelite_h::FADE_CROSSFADE as libc::c_int as libc::c_uint
                    {
                        if loglevel as libc::c_uint
                            >= crate::squeezelite_h::lINFO as libc::c_int as libc::c_uint
                        {
                            crate::src::utils::logprint(
                                b"%s %s:%d crossfade complete\n\x00" as *const u8
                                    as *const libc::c_char,
                                crate::src::utils::logtime(),
                                (*::std::mem::transmute::<&[u8; 15], &[libc::c_char; 15]>(
                                    b"_output_frames\x00",
                                ))
                                .as_ptr(),
                                200 as libc::c_int,
                            );
                        }
                        if crate::squeezelite_h::_buf_used(outputbuf)
                            >= dur_f.wrapping_mul(8 as libc::c_int as libc::c_uint)
                        {
                            crate::src::buffer::_buf_inc_readp(
                                outputbuf,
                                dur_f.wrapping_mul(8 as libc::c_int as libc::c_uint),
                            );
                            if loglevel as libc::c_uint
                                >= crate::squeezelite_h::lINFO as libc::c_int as libc::c_uint
                            {
                                crate::src::utils::logprint(
                                    b"%s %s:%d skipped crossfaded start\n\x00" as *const u8
                                        as *const libc::c_char,
                                    crate::src::utils::logtime(),
                                    (*::std::mem::transmute::<&[u8; 15], &[libc::c_char; 15]>(
                                        b"_output_frames\x00",
                                    ))
                                    .as_ptr(),
                                    203 as libc::c_int,
                                );
                            }
                        } else if loglevel as libc::c_uint
                            >= crate::squeezelite_h::lWARN as libc::c_int as libc::c_uint
                        {
                            crate::src::utils::logprint(
                                b"%s %s:%d unable to skip crossfaded start\n\x00" as *const u8
                                    as *const libc::c_char,
                                crate::src::utils::logtime(),
                                (*::std::mem::transmute::<&[u8; 15], &[libc::c_char; 15]>(
                                    b"_output_frames\x00",
                                ))
                                .as_ptr(),
                                205 as libc::c_int,
                            );
                        }
                        output.fade = crate::squeezelite_h::FADE_INACTIVE;
                        output.current_replay_gain = output.next_replay_gain
                    } else {
                        if loglevel as libc::c_uint
                            >= crate::squeezelite_h::lINFO as libc::c_int as libc::c_uint
                        {
                            crate::src::utils::logprint(
                                b"%s %s:%d fade complete\n\x00" as *const u8 as *const libc::c_char,
                                crate::src::utils::logtime(),
                                (*::std::mem::transmute::<&[u8; 15], &[libc::c_char; 15]>(
                                    b"_output_frames\x00",
                                ))
                                .as_ptr(),
                                210 as libc::c_int,
                            );
                        }
                        output.fade = crate::squeezelite_h::FADE_INACTIVE
                    }
                }
                // if fade in progress set fade gain, ensure cont_frames reduced so we get to end of fade at start of chunk
                if output.fade as u64 != 0 {
                    if output.fade_end > (*outputbuf).readp {
                        cont_frames_0 = if (cont_frames_0 as libc::c_long)
                            < output.fade_end.wrapping_offset_from((*outputbuf).readp)
                                as libc::c_long
                                / 8 as libc::c_int as libc::c_long
                        {
                            cont_frames_0 as libc::c_long
                        } else {
                            (output.fade_end.wrapping_offset_from((*outputbuf).readp)
                                as libc::c_long)
                                / 8 as libc::c_int as libc::c_long
                        } as crate::squeezelite_h::frames_t
                    }
                    if output.fade_dir as libc::c_uint
                        == crate::squeezelite_h::FADE_UP as libc::c_int as libc::c_uint
                        || output.fade_dir as libc::c_uint
                            == crate::squeezelite_h::FADE_DOWN as libc::c_int as libc::c_uint
                    {
                        // fade in, in-out, out handled via altering standard gain
                        let mut fade_gain: crate::squeezelite_h::s32_t = 0;
                        if output.fade_dir as libc::c_uint
                            == crate::squeezelite_h::FADE_DOWN as libc::c_int as libc::c_uint
                        {
                            cur_f = dur_f.wrapping_sub(cur_f)
                        }
                        fade_gain = crate::squeezelite_h::to_gain(
                            cur_f as libc::c_float / dur_f as libc::c_float,
                        );
                        gainL = crate::squeezelite_h::gain(gainL, fade_gain);
                        gainR = crate::squeezelite_h::gain(gainR, fade_gain);
                        if output.invert {
                            gainL = -gainL;
                            gainR = -gainR
                        }
                    }
                    if output.fade_dir as libc::c_uint
                        == crate::squeezelite_h::FADE_CROSS as libc::c_int as libc::c_uint
                    {
                        // cross fade requires special treatment - performed later based on these values
                        // support different replay gain for old and new track by retaining old value until crossfade completes
                        if crate::squeezelite_h::_buf_used(outputbuf)
                            .wrapping_div(8 as libc::c_int as libc::c_uint)
                            > dur_f.wrapping_add(size)
                        {
                            cross_gain_in = crate::squeezelite_h::to_gain(
                                cur_f as libc::c_float / dur_f as libc::c_float,
                            );
                            cross_gain_out = 0x10000 as libc::c_int - cross_gain_in;
                            if output.current_replay_gain != 0 {
                                cross_gain_out = crate::squeezelite_h::gain(
                                    cross_gain_out,
                                    output.current_replay_gain as crate::squeezelite_h::s32_t,
                                )
                            }
                            if output.next_replay_gain != 0 {
                                cross_gain_in = crate::squeezelite_h::gain(
                                    cross_gain_in,
                                    output.next_replay_gain as crate::squeezelite_h::s32_t,
                                )
                            }
                            gainL = output.gainL as crate::squeezelite_h::s32_t;
                            gainR = output.gainR as crate::squeezelite_h::s32_t;
                            if output.invert {
                                gainL = -gainL;
                                gainR = -gainR
                            }
                            cross_ptr =
                                output
                                    .fade_end
                                    .offset(cur_f.wrapping_mul(8 as libc::c_int as libc::c_uint)
                                        as isize)
                                    as *mut crate::squeezelite_h::s32_t
                        } else {
                            if loglevel as libc::c_uint
                                >= crate::squeezelite_h::lINFO as libc::c_int as libc::c_uint
                            {
                                crate::src::utils::logprint(
                                    b"%s %s:%d unable to continue crossfade - too few samples\n\x00"
                                        as *const u8
                                        as *const libc::c_char,
                                    crate::src::utils::logtime(),
                                    (*::std::mem::transmute::<&[u8; 15], &[libc::c_char; 15]>(
                                        b"_output_frames\x00",
                                    ))
                                    .as_ptr(),
                                    247 as libc::c_int,
                                );
                            }
                            output.fade = crate::squeezelite_h::FADE_INACTIVE
                        }
                    }
                }
            }
        }
        out_frames = if !silence {
            if size < cont_frames_0 {
                size
            } else {
                cont_frames_0
            }
        } else {
            size
        };
        wrote = output.write_cb.expect("non-null function pointer")(
            out_frames,
            silence,
            gainL,
            gainR,
            cross_gain_in,
            cross_gain_out,
            &mut cross_ptr,
        );
        if wrote <= 0 as libc::c_int {
            frames = (frames as libc::c_uint).wrapping_sub(size) as crate::squeezelite_h::frames_t
                as crate::squeezelite_h::frames_t;
            break;
        } else {
            out_frames = wrote as crate::squeezelite_h::frames_t;
            size = (size as libc::c_uint).wrapping_sub(out_frames) as crate::squeezelite_h::frames_t
                as crate::squeezelite_h::frames_t;
            if !silence {
                crate::src::buffer::_buf_inc_readp(
                    outputbuf,
                    out_frames.wrapping_mul(8 as libc::c_int as libc::c_uint),
                );
                output.frames_played = output.frames_played.wrapping_add(out_frames)
            }
        }
    }
    if loglevel as libc::c_uint >= crate::squeezelite_h::lSDEBUG as libc::c_int as libc::c_uint {
        crate::src::utils::logprint(
            b"%s %s:%d wrote %u frames\n\x00" as *const u8 as *const libc::c_char,
            crate::src::utils::logtime(),
            (*::std::mem::transmute::<&[u8; 15], &[libc::c_char; 15]>(b"_output_frames\x00"))
                .as_ptr(),
            276 as libc::c_int,
            frames,
        );
    }
    return frames;
}
// _* called with mutex locked
#[no_mangle]

pub unsafe extern "C" fn _checkfade(mut start: bool) {
    let mut bytes: crate::squeezelite_h::frames_t = 0;
    if loglevel as libc::c_uint >= crate::squeezelite_h::lINFO as libc::c_int as libc::c_uint {
        crate::src::utils::logprint(
            b"%s %s:%d fade mode: %u duration: %u %s\n\x00" as *const u8 as *const libc::c_char,
            crate::src::utils::logtime(),
            (*::std::mem::transmute::<&[u8; 11], &[libc::c_char; 11]>(b"_checkfade\x00")).as_ptr(),
            284 as libc::c_int,
            output.fade_mode as libc::c_uint,
            output.fade_secs,
            if start as libc::c_int != 0 {
                b"track-start\x00" as *const u8 as *const libc::c_char
            } else {
                b"track-end\x00" as *const u8 as *const libc::c_char
            },
        );
    }
    bytes = output
        .next_sample_rate
        .wrapping_mul(8 as libc::c_int as libc::c_uint)
        .wrapping_mul(output.fade_secs);
    if output.fade_mode as libc::c_uint
        == crate::squeezelite_h::FADE_INOUT as libc::c_int as libc::c_uint
    {
        /* align on a frame boundary */
        bytes = bytes
            .wrapping_div(2 as libc::c_int as libc::c_uint)
            .wrapping_div(8 as libc::c_int as libc::c_uint)
            .wrapping_mul(8 as libc::c_int as libc::c_uint)
    } // shorter than full buffer otherwise start and end align
    if start as libc::c_int != 0
        && (output.fade_mode as libc::c_uint
            == crate::squeezelite_h::FADE_IN as libc::c_int as libc::c_uint
            || output.fade_mode as libc::c_uint
                == crate::squeezelite_h::FADE_INOUT as libc::c_int as libc::c_uint
                && crate::squeezelite_h::_buf_used(outputbuf) == 0 as libc::c_int as libc::c_uint)
    {
        bytes = if (bytes as libc::c_ulong)
            < (*outputbuf)
                .size
                .wrapping_sub(8 as libc::c_int as libc::c_ulong)
        {
            bytes as libc::c_ulong
        } else {
            (*outputbuf)
                .size
                .wrapping_sub(8 as libc::c_int as libc::c_ulong)
        } as crate::squeezelite_h::frames_t; // max of current remaining samples from previous track
        if loglevel as libc::c_uint >= crate::squeezelite_h::lINFO as libc::c_int as libc::c_uint {
            crate::src::utils::logprint(
                b"%s %s:%d fade IN: %u frames\n\x00" as *const u8 as *const libc::c_char,
                crate::src::utils::logtime(),
                (*::std::mem::transmute::<&[u8; 11], &[libc::c_char; 11]>(b"_checkfade\x00"))
                    .as_ptr(),
                294 as libc::c_int,
                bytes.wrapping_div(8 as libc::c_int as libc::c_uint),
            ); // max of 90% of outputbuf as we consume additional buffer during crossfade
        }
        output.fade = crate::squeezelite_h::FADE_DUE;
        output.fade_dir = crate::squeezelite_h::FADE_UP;
        output.fade_start = (*outputbuf).writep;
        output.fade_end = output.fade_start.offset(bytes as isize);
        if output.fade_end >= (*outputbuf).wrap {
            output.fade_end = output.fade_end.offset(-((*outputbuf).size as isize))
        }
    }
    if !start
        && (output.fade_mode as libc::c_uint
            == crate::squeezelite_h::FADE_OUT as libc::c_int as libc::c_uint
            || output.fade_mode as libc::c_uint
                == crate::squeezelite_h::FADE_INOUT as libc::c_int as libc::c_uint)
    {
        bytes = if crate::squeezelite_h::_buf_used(outputbuf) < bytes {
            crate::squeezelite_h::_buf_used(outputbuf)
        } else {
            bytes
        };
        if loglevel as libc::c_uint >= crate::squeezelite_h::lINFO as libc::c_int as libc::c_uint {
            crate::src::utils::logprint(
                b"%s %s:%d fade %s: %u frames\n\x00" as *const u8 as *const libc::c_char,
                crate::src::utils::logtime(),
                (*::std::mem::transmute::<&[u8; 11], &[libc::c_char; 11]>(b"_checkfade\x00"))
                    .as_ptr(),
                306 as libc::c_int,
                if output.fade_mode as libc::c_uint
                    == crate::squeezelite_h::FADE_INOUT as libc::c_int as libc::c_uint
                {
                    b"IN-OUT\x00" as *const u8 as *const libc::c_char
                } else {
                    b"OUT\x00" as *const u8 as *const libc::c_char
                },
                bytes.wrapping_div(8 as libc::c_int as libc::c_uint),
            );
        }
        output.fade = crate::squeezelite_h::FADE_DUE;
        output.fade_dir = crate::squeezelite_h::FADE_DOWN;
        output.fade_start = (*outputbuf).writep.offset(-(bytes as isize));
        if output.fade_start < (*outputbuf).buf {
            output.fade_start = output.fade_start.offset((*outputbuf).size as isize)
        }
        output.fade_end = (*outputbuf).writep
    }
    if start as libc::c_int != 0
        && output.fade_mode as libc::c_uint
            == crate::squeezelite_h::FADE_CROSSFADE as libc::c_int as libc::c_uint
    {
        if crate::squeezelite_h::_buf_used(outputbuf) != 0 as libc::c_int as libc::c_uint {
            if output.next_sample_rate != output.current_sample_rate {
                if loglevel as libc::c_uint
                    >= crate::squeezelite_h::lINFO as libc::c_int as libc::c_uint
                {
                    crate::src::utils::logprint(
                        b"%s %s:%d crossfade disabled as sample rates differ\n\x00" as *const u8
                            as *const libc::c_char,
                        crate::src::utils::logtime(),
                        (*::std::mem::transmute::<&[u8; 11], &[libc::c_char; 11]>(
                            b"_checkfade\x00",
                        ))
                        .as_ptr(),
                        319 as libc::c_int,
                    );
                }
                return;
            }
            bytes = if bytes < crate::squeezelite_h::_buf_used(outputbuf) {
                bytes
            } else {
                crate::squeezelite_h::_buf_used(outputbuf)
            };
            bytes = if bytes
                < ((*outputbuf).size as libc::c_double * 0.9f64) as crate::squeezelite_h::frames_t
            {
                bytes
            } else {
                ((*outputbuf).size as libc::c_double * 0.9f64) as crate::squeezelite_h::frames_t
            };
            if loglevel as libc::c_uint
                >= crate::squeezelite_h::lINFO as libc::c_int as libc::c_uint
            {
                crate::src::utils::logprint(
                    b"%s %s:%d CROSSFADE: %u frames\n\x00" as *const u8 as *const libc::c_char,
                    crate::src::utils::logtime(),
                    (*::std::mem::transmute::<&[u8; 11], &[libc::c_char; 11]>(b"_checkfade\x00"))
                        .as_ptr(),
                    324 as libc::c_int,
                    bytes.wrapping_div(8 as libc::c_int as libc::c_uint),
                );
            }
            output.fade = crate::squeezelite_h::FADE_DUE;
            output.fade_dir = crate::squeezelite_h::FADE_CROSS;
            output.fade_start = (*outputbuf).writep.offset(-(bytes as isize));
            if output.fade_start < (*outputbuf).buf {
                output.fade_start = output.fade_start.offset((*outputbuf).size as isize)
            }
            output.fade_end = (*outputbuf).writep;
            output.track_start = output.fade_start
        } else if (*outputbuf).size
            == (44100 as libc::c_int * 8 as libc::c_int * 10 as libc::c_int) as libc::c_ulong
            && (*outputbuf).readp == (*outputbuf).buf
        {
            // if default setting used and nothing in buffer attempt to resize to provide full crossfade support
            if loglevel as libc::c_uint
                >= crate::squeezelite_h::lINFO as libc::c_int as libc::c_uint
            {
                crate::src::utils::logprint(
                    b"%s %s:%d resize outputbuf for crossfade\n\x00" as *const u8
                        as *const libc::c_char,
                    crate::src::utils::logtime(),
                    (*::std::mem::transmute::<&[u8; 11], &[libc::c_char; 11]>(b"_checkfade\x00"))
                        .as_ptr(),
                    335 as libc::c_int,
                );
            }
            crate::src::buffer::_buf_resize(
                outputbuf,
                (44100 as libc::c_int * 8 as libc::c_int * 10 as libc::c_int * 12 as libc::c_int
                    / 10 as libc::c_int) as crate::stddef_h::size_t,
            );
            crate::src::utils::touch_memory((*outputbuf).buf, (*outputbuf).size);
        }
    };
}
#[no_mangle]

pub unsafe extern "C" fn output_init_common(
    mut level: crate::squeezelite_h::log_level,
    mut device: *const libc::c_char,
    mut output_buf_size: libc::c_uint,
    mut rates: *mut libc::c_uint,
    mut idle: libc::c_uint,
) {
    let mut i: libc::c_uint = 0;
    loglevel = level;
    output_buf_size = output_buf_size
        .wrapping_sub(output_buf_size.wrapping_rem(8 as libc::c_int as libc::c_uint));
    if loglevel as libc::c_uint >= crate::squeezelite_h::lDEBUG as libc::c_int as libc::c_uint {
        crate::src::utils::logprint(
            b"%s %s:%d outputbuf size: %u\n\x00" as *const u8 as *const libc::c_char,
            crate::src::utils::logtime(),
            (*::std::mem::transmute::<&[u8; 19], &[libc::c_char; 19]>(b"output_init_common\x00"))
                .as_ptr(),
            350 as libc::c_int,
            output_buf_size,
        );
    }
    crate::src::buffer::buf_init(outputbuf, output_buf_size as crate::stddef_h::size_t);
    if (*outputbuf).buf.is_null() {
        crate::src::utils::logprint(
            b"%s %s:%d unable to malloc output buffer\n\x00" as *const u8 as *const libc::c_char,
            crate::src::utils::logtime(),
            (*::std::mem::transmute::<&[u8; 19], &[libc::c_char; 19]>(b"output_init_common\x00"))
                .as_ptr(),
            354 as libc::c_int,
        );
        crate::stdlib::exit(0 as libc::c_int);
    }
    silencebuf = crate::stdlib::malloc((2048 as libc::c_int * 8 as libc::c_int) as libc::c_ulong)
        as *mut crate::squeezelite_h::u8_t;
    if silencebuf.is_null() {
        crate::src::utils::logprint(
            b"%s %s:%d unable to malloc silence buffer\n\x00" as *const u8 as *const libc::c_char,
            crate::src::utils::logtime(),
            (*::std::mem::transmute::<&[u8; 19], &[libc::c_char; 19]>(b"output_init_common\x00"))
                .as_ptr(),
            360 as libc::c_int,
        );
        crate::stdlib::exit(0 as libc::c_int);
    }
    crate::stdlib::memset(
        silencebuf as *mut libc::c_void,
        0 as libc::c_int,
        (2048 as libc::c_int * 8 as libc::c_int) as libc::c_ulong,
    );
    if loglevel as libc::c_uint >= crate::squeezelite_h::lDEBUG as libc::c_int as libc::c_uint {
        crate::src::utils::logprint(
            b"%s %s:%d idle timeout: %u\n\x00" as *const u8 as *const libc::c_char,
            crate::src::utils::logtime(),
            (*::std::mem::transmute::<&[u8; 19], &[libc::c_char; 19]>(b"output_init_common\x00"))
                .as_ptr(),
            374 as libc::c_int,
            idle,
        );
    }
    output.state = if idle != 0 {
        crate::squeezelite_h::OUTPUT_OFF as libc::c_int
    } else {
        crate::squeezelite_h::OUTPUT_STOPPED as libc::c_int
    } as crate::squeezelite_h::output_state;
    output.device = device;
    output.fade = crate::squeezelite_h::FADE_INACTIVE;
    output.invert = 0 as libc::c_int != 0;
    output.error_opening = 0 as libc::c_int != 0;
    output.idle_to = idle;
    /* Skip test_open for stdout, set default sample rates */
    if *output.device.offset(0 as libc::c_int as isize) as libc::c_int == '-' as i32 {
        i = 0 as libc::c_int as libc::c_uint;
        while i < 18 as libc::c_int as libc::c_uint {
            output.supported_rates[i as usize] = *rates.offset(i as isize);
            i = i.wrapping_add(1)
        }
    } else if !crate::src::output_alsa::test_open(
        output.device,
        output.supported_rates.as_mut_ptr(),
        user_rates,
    ) {
        crate::src::utils::logprint(
            b"%s %s:%d unable to open output device: %s\n\x00" as *const u8 as *const libc::c_char,
            crate::src::utils::logtime(),
            (*::std::mem::transmute::<&[u8; 19], &[libc::c_char; 19]>(b"output_init_common\x00"))
                .as_ptr(),
            391 as libc::c_int,
            output.device,
        );
        crate::stdlib::exit(0 as libc::c_int);
    }
    if user_rates {
        i = 0 as libc::c_int as libc::c_uint;
        while i < 18 as libc::c_int as libc::c_uint {
            output.supported_rates[i as usize] = *rates.offset(i as isize);
            i = i.wrapping_add(1)
        }
    }
    // set initial sample rate, preferring 44100
    i = 0 as libc::c_int as libc::c_uint;
    while i < 18 as libc::c_int as libc::c_uint {
        if output.supported_rates[i as usize] == 44100 as libc::c_int as libc::c_uint {
            output.default_sample_rate = 44100 as libc::c_int as libc::c_uint;
            break;
        } else {
            i = i.wrapping_add(1)
        }
    }
    if output.default_sample_rate == 0 {
        output.default_sample_rate = output.supported_rates[0 as libc::c_int as usize]
    }
    output.current_sample_rate = output.default_sample_rate;
    if loglevel as libc::c_uint >= crate::squeezelite_h::lINFO as libc::c_int as libc::c_uint {
        let mut rates_buf: [libc::c_char; 180] =
            *::std::mem::transmute::<&[u8; 180],
                                     &mut [libc::c_char; 180]>(b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00");
        i = 0 as libc::c_int as libc::c_uint;
        while output.supported_rates[i as usize] != 0 {
            let mut s: [libc::c_char; 10] = [0; 10];
            crate::stdlib::sprintf(
                s.as_mut_ptr(),
                b"%d \x00" as *const u8 as *const libc::c_char,
                output.supported_rates[i as usize],
            );
            crate::stdlib::strcat(rates_buf.as_mut_ptr(), s.as_mut_ptr());
            i = i.wrapping_add(1)
        }
        if loglevel as libc::c_uint >= crate::squeezelite_h::lINFO as libc::c_int as libc::c_uint {
            crate::src::utils::logprint(
                b"%s %s:%d supported rates: %s\n\x00" as *const u8 as *const libc::c_char,
                crate::src::utils::logtime(),
                (*::std::mem::transmute::<&[u8; 19], &[libc::c_char; 19]>(
                    b"output_init_common\x00",
                ))
                .as_ptr(),
                422 as libc::c_int,
                rates_buf.as_mut_ptr(),
            );
        }
    };
}
#[no_mangle]

pub unsafe extern "C" fn output_close_common() {
    crate::src::buffer::buf_destroy(outputbuf);
    crate::stdlib::free(silencebuf as *mut libc::c_void);
}
/*
 *  Squeezelite - lightweight headless squeezebox emulator
 *
 *  (c) Adrian Smith 2012-2015, triode1@btinternet.com
 *      Ralph Irving 2015-2017, ralph_irving@hotmail.com
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additions (c) Paul Hermann, 2015-2017 under the same license terms
 *   -Control of Raspberry pi GPIO for amplifier power
 *   -Launch script on power status change from LMS
 */
// make may define: PORTAUDIO, SELFPIPE, RESAMPLE, RESAMPLE_MP, VISEXPORT, GPIO, IR, DSD, LINKALL to influence build
// build detection
// dynamically loaded libraries at run time
// !LINKALL
// config options
// do not reduce as icy-meta max is 4080
/* SUN */
/* SUN */
// printf/scanf formats for u64_t
// logging
// utils.c (non logging)
// buffer.c
// _* called with mutex locked
// slimproto.c
// stream.c
// decode.c
// output.c output_alsa.c output_pa.c output_pack.c
// frames played at the point delay is measured
// ordered largest first so [0] is max_rate
// set in decode thread
// set in decode thread
// set by slimproto
// set by slimproto
// set by slimproto
// set by slimproto
// set by slimproto
// set by slimproto
// set by slimproto
#[no_mangle]

pub unsafe extern "C" fn output_flush() {
    if loglevel as libc::c_uint >= crate::squeezelite_h::lINFO as libc::c_int as libc::c_uint {
        crate::src::utils::logprint(
            b"%s %s:%d flush output buffer\n\x00" as *const u8 as *const libc::c_char,
            crate::src::utils::logtime(),
            (*::std::mem::transmute::<&[u8; 13], &[libc::c_char; 13]>(b"output_flush\x00"))
                .as_ptr(),
            435 as libc::c_int,
        );
    }
    crate::src::buffer::buf_flush(outputbuf);
    crate::stdlib::pthread_mutex_lock(&mut (*outputbuf).mutex);
    output.fade = crate::squeezelite_h::FADE_INACTIVE;
    if output.state as libc::c_int != crate::squeezelite_h::OUTPUT_OFF as libc::c_int {
        output.state = crate::squeezelite_h::OUTPUT_STOPPED;
        if output.error_opening {
            output.current_sample_rate = output.default_sample_rate
        }
        output.delay_active = 0 as libc::c_int != 0
    }
    output.frames_played = 0 as libc::c_int as libc::c_uint;
    crate::stdlib::pthread_mutex_unlock(&mut (*outputbuf).mutex);
}
