use ::libc;

pub use crate::squeezelite_h::frames_t;
pub use crate::squeezelite_h::output_format;
pub use crate::squeezelite_h::s32_t;
pub use crate::squeezelite_h::s64_t;
pub use crate::squeezelite_h::u32_t;
pub use crate::squeezelite_h::u8_t;
pub use crate::squeezelite_h::S16_LE;
pub use crate::squeezelite_h::S24_3LE;
pub use crate::squeezelite_h::S24_LE;
pub use crate::squeezelite_h::S32_LE;
pub use crate::stdlib::__int32_t;
pub use crate::stdlib::__int64_t;
pub use crate::stdlib::__uint32_t;
pub use crate::stdlib::__uint8_t;
pub use crate::stdlib::int32_t;
pub use crate::stdlib::int64_t;
use crate::stdlib::memcpy;
pub use crate::stdlib::u_int32_t;
pub use crate::stdlib::u_int8_t;
pub use crate::stdlib::uintptr_t;
// inlining these on windows prevents them being linkable...
#[inline]

unsafe extern "C" fn gain(
    mut gain_0: crate::squeezelite_h::s32_t,
    mut sample: crate::squeezelite_h::s32_t,
) -> crate::squeezelite_h::s32_t {
    let mut res = gain_0 as crate::squeezelite_h::s64_t * sample as crate::squeezelite_h::s64_t;
    if res as libc::c_longlong > 0x7fffffffffff as libc::c_longlong {
        res = 0x7fffffffffff as libc::c_longlong as crate::squeezelite_h::s64_t
    }
    if (res as libc::c_longlong) < -(0x7fffffffffff as libc::c_longlong) {
        res = -(0x7fffffffffff as libc::c_longlong) as crate::squeezelite_h::s64_t
    }
    return (res >> 16 as libc::c_int) as crate::squeezelite_h::s32_t;
}
#[no_mangle]

pub unsafe extern "C" fn _scale_and_pack_frames(
    mut outputptr: *mut libc::c_void,
    mut inputptr: *mut crate::squeezelite_h::s32_t,
    mut cnt: crate::squeezelite_h::frames_t,
    mut gainL: crate::squeezelite_h::s32_t,
    mut gainR: crate::squeezelite_h::s32_t,
    mut format: crate::squeezelite_h::output_format,
) {
    match format as libc::c_uint {
        3 => {
            let mut optr = outputptr as *mut crate::squeezelite_h::u32_t;
            if gainL == 0x10000 as libc::c_int && gainR == 0x10000 as libc::c_int {
                loop {
                    let fresh0 = cnt;
                    cnt = cnt.wrapping_sub(1);
                    if !(fresh0 != 0) {
                        break;
                    }
                    let fresh1 = optr;
                    optr = optr.offset(1);
                    *fresh1 = (*inputptr >> 16 as libc::c_int & 0xffff as libc::c_int)
                        as libc::c_uint
                        | *inputptr.offset(1 as libc::c_int as isize) as libc::c_uint
                            & 0xffff0000 as libc::c_uint;
                    inputptr = inputptr.offset(2 as libc::c_int as isize)
                }
            } else {
                loop {
                    let fresh2 = cnt;
                    cnt = cnt.wrapping_sub(1);
                    if !(fresh2 != 0) {
                        break;
                    }
                    let fresh3 = optr;
                    optr = optr.offset(1);
                    *fresh3 = (gain(gainL, *inputptr) >> 16 as libc::c_int & 0xffff as libc::c_int)
                        as libc::c_uint
                        | gain(gainR, *inputptr.offset(1 as libc::c_int as isize)) as libc::c_uint
                            & 0xffff0000 as libc::c_uint;
                    inputptr = inputptr.offset(2 as libc::c_int as isize)
                }
            }
        }
        1 => {
            let mut optr_0 = outputptr as *mut crate::squeezelite_h::u32_t;
            if gainL == 0x10000 as libc::c_int && gainR == 0x10000 as libc::c_int {
                loop {
                    let fresh4 = cnt;
                    cnt = cnt.wrapping_sub(1);
                    if !(fresh4 != 0) {
                        break;
                    }
                    let fresh5 = inputptr;
                    inputptr = inputptr.offset(1);
                    let fresh6 = optr_0;
                    optr_0 = optr_0.offset(1);
                    *fresh6 = (*fresh5 >> 8 as libc::c_int) as crate::squeezelite_h::u32_t;
                    let fresh7 = inputptr;
                    inputptr = inputptr.offset(1);
                    let fresh8 = optr_0;
                    optr_0 = optr_0.offset(1);
                    *fresh8 = (*fresh7 >> 8 as libc::c_int) as crate::squeezelite_h::u32_t
                }
            } else {
                loop {
                    let fresh9 = cnt;
                    cnt = cnt.wrapping_sub(1);
                    if !(fresh9 != 0) {
                        break;
                    }
                    let fresh10 = inputptr;
                    inputptr = inputptr.offset(1);
                    let fresh11 = optr_0;
                    optr_0 = optr_0.offset(1);
                    *fresh11 =
                        (gain(gainL, *fresh10) >> 8 as libc::c_int) as crate::squeezelite_h::u32_t;
                    let fresh12 = inputptr;
                    inputptr = inputptr.offset(1);
                    let fresh13 = optr_0;
                    optr_0 = optr_0.offset(1);
                    *fresh13 =
                        (gain(gainR, *fresh12) >> 8 as libc::c_int) as crate::squeezelite_h::u32_t
                }
            }
        }
        2 => {
            let mut optr_1 = outputptr as *mut crate::squeezelite_h::u8_t;
            if gainL == 0x10000 as libc::c_int && gainR == 0x10000 as libc::c_int {
                while cnt != 0 {
                    // attempt to do 32 bit memory accesses - move 2 frames at once: 16 bytes -> 12 bytes
                    // falls through to exception case when not aligned or if less than 2 frames to move
                    if optr_1 as crate::stdlib::uintptr_t & 0x3 as libc::c_int as libc::c_ulong
                        == 0 as libc::c_int as libc::c_ulong
                        && cnt >= 2 as libc::c_int as libc::c_uint
                    {
                        let mut o_ptr =
                            optr_1 as *mut libc::c_void as *mut crate::squeezelite_h::u32_t;
                        while cnt >= 2 as libc::c_int as libc::c_uint {
                            let fresh14 = inputptr;
                            inputptr = inputptr.offset(1);
                            let mut l1 = *fresh14;
                            let fresh15 = inputptr;
                            inputptr = inputptr.offset(1);
                            let mut r1 = *fresh15;
                            let fresh16 = inputptr;
                            inputptr = inputptr.offset(1);
                            let mut l2 = *fresh16;
                            let fresh17 = inputptr;
                            inputptr = inputptr.offset(1);
                            let mut r2 = *fresh17;
                            let fresh18 = o_ptr;
                            o_ptr = o_ptr.offset(1);
                            *fresh18 = (l1 as libc::c_uint & 0xffffff00 as libc::c_uint)
                                >> 8 as libc::c_int
                                | ((r1 & 0xff00 as libc::c_int) << 16 as libc::c_int)
                                    as libc::c_uint;
                            let fresh19 = o_ptr;
                            o_ptr = o_ptr.offset(1);
                            *fresh19 = (r1 as libc::c_uint & 0xffff0000 as libc::c_uint)
                                >> 16 as libc::c_int
                                | ((l2 & 0xffff00 as libc::c_int) << 8 as libc::c_int)
                                    as libc::c_uint;
                            let fresh20 = o_ptr;
                            o_ptr = o_ptr.offset(1);
                            *fresh20 = (l2 as libc::c_uint & 0xff000000 as libc::c_uint)
                                >> 24 as libc::c_int
                                | r2 as libc::c_uint & 0xffffff00 as libc::c_uint;
                            optr_1 = optr_1.offset(12 as libc::c_int as isize);
                            cnt = (cnt as libc::c_uint)
                                .wrapping_sub(2 as libc::c_int as libc::c_uint)
                                as crate::squeezelite_h::frames_t
                                as crate::squeezelite_h::frames_t
                        }
                    } else {
                        let fresh21 = inputptr;
                        inputptr = inputptr.offset(1);
                        let mut lsample = *fresh21;
                        let fresh22 = inputptr;
                        inputptr = inputptr.offset(1);
                        let mut rsample = *fresh22;
                        let fresh23 = optr_1;
                        optr_1 = optr_1.offset(1);
                        *fresh23 = ((lsample & 0xff00 as libc::c_int) >> 8 as libc::c_int)
                            as crate::squeezelite_h::u8_t;
                        let fresh24 = optr_1;
                        optr_1 = optr_1.offset(1);
                        *fresh24 = ((lsample & 0xff0000 as libc::c_int) >> 16 as libc::c_int)
                            as crate::squeezelite_h::u8_t;
                        let fresh25 = optr_1;
                        optr_1 = optr_1.offset(1);
                        *fresh25 = ((lsample as libc::c_uint & 0xff000000 as libc::c_uint)
                            >> 24 as libc::c_int)
                            as crate::squeezelite_h::u8_t;
                        let fresh26 = optr_1;
                        optr_1 = optr_1.offset(1);
                        *fresh26 = ((rsample & 0xff00 as libc::c_int) >> 8 as libc::c_int)
                            as crate::squeezelite_h::u8_t;
                        let fresh27 = optr_1;
                        optr_1 = optr_1.offset(1);
                        *fresh27 = ((rsample & 0xff0000 as libc::c_int) >> 16 as libc::c_int)
                            as crate::squeezelite_h::u8_t;
                        let fresh28 = optr_1;
                        optr_1 = optr_1.offset(1);
                        *fresh28 = ((rsample as libc::c_uint & 0xff000000 as libc::c_uint)
                            >> 24 as libc::c_int)
                            as crate::squeezelite_h::u8_t;
                        cnt = cnt.wrapping_sub(1)
                    }
                }
            } else {
                while cnt != 0 {
                    // attempt to do 32 bit memory accesses - move 2 frames at once: 16 bytes -> 12 bytes
                    // falls through to exception case when not aligned or if less than 2 frames to move
                    if optr_1 as crate::stdlib::uintptr_t & 0x3 as libc::c_int as libc::c_ulong
                        == 0 as libc::c_int as libc::c_ulong
                        && cnt >= 2 as libc::c_int as libc::c_uint
                    {
                        let mut o_ptr_0 =
                            optr_1 as *mut libc::c_void as *mut crate::squeezelite_h::u32_t;
                        while cnt >= 2 as libc::c_int as libc::c_uint {
                            let fresh29 = inputptr;
                            inputptr = inputptr.offset(1);
                            let mut l1_0 = gain(gainL, *fresh29);
                            let fresh30 = inputptr;
                            inputptr = inputptr.offset(1);
                            let mut r1_0 = gain(gainR, *fresh30);
                            let fresh31 = inputptr;
                            inputptr = inputptr.offset(1);
                            let mut l2_0 = gain(gainL, *fresh31);
                            let fresh32 = inputptr;
                            inputptr = inputptr.offset(1);
                            let mut r2_0 = gain(gainR, *fresh32);
                            let fresh33 = o_ptr_0;
                            o_ptr_0 = o_ptr_0.offset(1);
                            *fresh33 = (l1_0 as libc::c_uint & 0xffffff00 as libc::c_uint)
                                >> 8 as libc::c_int
                                | ((r1_0 & 0xff00 as libc::c_int) << 16 as libc::c_int)
                                    as libc::c_uint;
                            let fresh34 = o_ptr_0;
                            o_ptr_0 = o_ptr_0.offset(1);
                            *fresh34 = (r1_0 as libc::c_uint & 0xffff0000 as libc::c_uint)
                                >> 16 as libc::c_int
                                | ((l2_0 & 0xffff00 as libc::c_int) << 8 as libc::c_int)
                                    as libc::c_uint;
                            let fresh35 = o_ptr_0;
                            o_ptr_0 = o_ptr_0.offset(1);
                            *fresh35 = (l2_0 as libc::c_uint & 0xff000000 as libc::c_uint)
                                >> 24 as libc::c_int
                                | r2_0 as libc::c_uint & 0xffffff00 as libc::c_uint;
                            optr_1 = optr_1.offset(12 as libc::c_int as isize);
                            cnt = (cnt as libc::c_uint)
                                .wrapping_sub(2 as libc::c_int as libc::c_uint)
                                as crate::squeezelite_h::frames_t
                                as crate::squeezelite_h::frames_t
                        }
                    } else {
                        let fresh36 = inputptr;
                        inputptr = inputptr.offset(1);
                        let mut lsample_0 = gain(gainL, *fresh36);
                        let fresh37 = inputptr;
                        inputptr = inputptr.offset(1);
                        let mut rsample_0 = gain(gainR, *fresh37);
                        let fresh38 = optr_1;
                        optr_1 = optr_1.offset(1);
                        *fresh38 = ((lsample_0 & 0xff00 as libc::c_int) >> 8 as libc::c_int)
                            as crate::squeezelite_h::u8_t;
                        let fresh39 = optr_1;
                        optr_1 = optr_1.offset(1);
                        *fresh39 = ((lsample_0 & 0xff0000 as libc::c_int) >> 16 as libc::c_int)
                            as crate::squeezelite_h::u8_t;
                        let fresh40 = optr_1;
                        optr_1 = optr_1.offset(1);
                        *fresh40 = ((lsample_0 as libc::c_uint & 0xff000000 as libc::c_uint)
                            >> 24 as libc::c_int)
                            as crate::squeezelite_h::u8_t;
                        let fresh41 = optr_1;
                        optr_1 = optr_1.offset(1);
                        *fresh41 = ((rsample_0 & 0xff00 as libc::c_int) >> 8 as libc::c_int)
                            as crate::squeezelite_h::u8_t;
                        let fresh42 = optr_1;
                        optr_1 = optr_1.offset(1);
                        *fresh42 = ((rsample_0 & 0xff0000 as libc::c_int) >> 16 as libc::c_int)
                            as crate::squeezelite_h::u8_t;
                        let fresh43 = optr_1;
                        optr_1 = optr_1.offset(1);
                        *fresh43 = ((rsample_0 as libc::c_uint & 0xff000000 as libc::c_uint)
                            >> 24 as libc::c_int)
                            as crate::squeezelite_h::u8_t;
                        cnt = cnt.wrapping_sub(1)
                    }
                }
            }
        }
        0 => {
            let mut optr_2 = outputptr as *mut crate::squeezelite_h::u32_t;
            if gainL == 0x10000 as libc::c_int && gainR == 0x10000 as libc::c_int {
                crate::stdlib::memcpy(
                    outputptr,
                    inputptr as *const libc::c_void,
                    cnt.wrapping_mul(8 as libc::c_int as libc::c_uint) as libc::c_ulong,
                );
            } else {
                loop {
                    let fresh44 = cnt;
                    cnt = cnt.wrapping_sub(1);
                    if !(fresh44 != 0) {
                        break;
                    }
                    let fresh45 = inputptr;
                    inputptr = inputptr.offset(1);
                    let fresh46 = optr_2;
                    optr_2 = optr_2.offset(1);
                    *fresh46 = gain(gainL, *fresh45) as crate::squeezelite_h::u32_t;
                    let fresh47 = inputptr;
                    inputptr = inputptr.offset(1);
                    let fresh48 = optr_2;
                    optr_2 = optr_2.offset(1);
                    *fresh48 = gain(gainR, *fresh47) as crate::squeezelite_h::u32_t
                }
            }
        }
        _ => {}
    };
}
